
from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x9EEaBBcf42F0c4900d302544Cce599811C2De2b9",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "cancelDuelEntry", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setDuelCompleteBlocks", "type": "function", "inputs": [{"name": "_blocks", "type": "uint16", "internalType": "uint16"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setGoldPot", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setLoseWinThresholds", "type": "function", "inputs": [{"name": "_lose", "type": "uint64", "internalType": "uint64"}, {"name": "_win", "type": "uint64", "internalType": "uint64"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setMatchLimit", "type": "function", "inputs": [{"name": "_limit", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setMatchMaker", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}, {"name": "_access", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setMatchThreshold", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}, {"name": "_threshold", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setModerator", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}, {"name": "_access", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setOpenMatchmaking", "type": "function", "inputs": [{"name": "_open", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setScore", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_score", "type": "uint64", "internalType": "uint64"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setSeasonStartAndDuration", "type": "function", "inputs": [{"name": "_start", "type": "uint256", "internalType": "uint256"}, {"name": "_duration", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStatScienceAddress", "type": "function", "inputs": [{"name": "_statScienceAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTimeoutThreshold", "type": "function", "inputs": [{"name": "_threshold", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTypeAllowed", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}, {"name": "_allowed", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "completeDuel", "type": "function", "inputs": [{"name": "_duelId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getCurrentClassBonuses", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8[6][2]", "internalType": "uint8[6][2]"}], "stateMutability": "view"},
    {"name": "completeDuelAdmin", "type": "function", "inputs": [{"name": "_duelId", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getKeyExists", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_key", "type": "uint256", "internalType": "uint256"}, {"name": "_value", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "enterDuelLobby", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "_background", "type": "uint8", "internalType": "uint8"}, {"name": "_stat", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "DuelCompleted", "type": "event", "inputs": [{"name": "duelId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player1", "type": "address", "indexed": true, "internalType": "address"}, {"name": "player2", "type": "address", "indexed": true, "internalType": "address"}, {"name": "duel", "type": "tuple", "indexed": false, "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel"}], "anonymous": false},
    {"name": "DuelCreated", "type": "event", "inputs": [{"name": "duelId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player1", "type": "address", "indexed": true, "internalType": "address"}, {"name": "player2", "type": "address", "indexed": true, "internalType": "address"}, {"name": "duel", "type": "tuple", "indexed": false, "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel"}], "anonymous": false},
    {"name": "DuelEntryCreated", "type": "event", "inputs": [{"name": "id", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "heroIds", "type": "uint256[]", "indexed": false, "internalType": "uint256[]"}], "anonymous": false},
    {"name": "DuelEntryMatched", "type": "event", "inputs": [{"name": "duelId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "duelEntryId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player1", "type": "address", "indexed": true, "internalType": "address"}, {"name": "player2", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "PlayerScoreChanged", "type": "event", "inputs": [{"name": "duelId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "playerScoreChange", "type": "tuple", "indexed": false, "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "anonymous": false},
    {"name": "TurnOutcome", "type": "event", "inputs": [{"name": "duelId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player1HeroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "player2HeroId", "type": "uint256", "indexed": true, "internalType": "uint256"}, {"name": "turnResult", "type": "tuple", "indexed": false, "components": [{"name": "turn", "type": "uint16", "internalType": "uint16"}, {"name": "player1HeroId", "type": "uint256", "internalType": "uint256"}, {"name": "player2HeroId", "type": "uint256", "internalType": "uint256"}, {"name": "stat", "type": "uint8", "internalType": "uint8"}, {"name": "background", "type": "uint8", "internalType": "uint8"}, {"name": "hero1Score", "type": "tuple", "components": [{"name": "roll", "type": "uint8", "internalType": "uint8"}, {"name": "elementBonus", "type": "uint16", "internalType": "uint16"}, {"name": "statValue", "type": "uint16", "internalType": "uint16"}, {"name": "backgroundBonus", "type": "uint16", "internalType": "uint16"}, {"name": "classBonus", "type": "uint16", "internalType": "uint16"}, {"name": "subclassBonus", "type": "uint16", "internalType": "uint16"}, {"name": "total", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IDuelS2Types.HeroTurnScore"}, {"name": "hero2Score", "type": "tuple", "components": [{"name": "roll", "type": "uint8", "internalType": "uint8"}, {"name": "elementBonus", "type": "uint16", "internalType": "uint16"}, {"name": "statValue", "type": "uint16", "internalType": "uint16"}, {"name": "backgroundBonus", "type": "uint16", "internalType": "uint16"}, {"name": "classBonus", "type": "uint16", "internalType": "uint16"}, {"name": "subclassBonus", "type": "uint16", "internalType": "uint16"}, {"name": "total", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IDuelS2Types.HeroTurnScore"}, {"name": "winnerHeroId", "type": "uint256", "internalType": "uint256"}, {"name": "winnerPlayer", "type": "address", "internalType": "address"}], "internalType": "struct IDuelS2Types.TurnResult"}], "anonymous": false},
    {"name": "getActiveDuels", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel[]"}], "stateMutability": "view"},
    {"name": "getChallenges", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel[]"}], "stateMutability": "view"},
    {"name": "getCurrentHeroScoreDuelId", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getDuel", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel"}], "stateMutability": "view"},
    {"name": "getDuelEntry", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getDuelHistory", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[100]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel[100]"}], "stateMutability": "view"},
    {"name": "getDuelIndexP1", "type": "function", "inputs": [{"name": "_duelId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getDuelRewards", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "item", "type": "address", "internalType": "address"}, {"name": "recipient", "type": "address", "internalType": "address"}, {"name": "qty", "type": "uint256", "internalType": "uint256"}], "internalType": "struct IDuelS2Types.DuelReward[]"}], "stateMutability": "view"},
    {"name": "getDuelTurns", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "turn", "type": "uint16", "internalType": "uint16"}, {"name": "player1HeroId", "type": "uint256", "internalType": "uint256"}, {"name": "player2HeroId", "type": "uint256", "internalType": "uint256"}, {"name": "stat", "type": "uint8", "internalType": "uint8"}, {"name": "background", "type": "uint8", "internalType": "uint8"}, {"name": "hero1Score", "type": "tuple", "components": [{"name": "roll", "type": "uint8", "internalType": "uint8"}, {"name": "elementBonus", "type": "uint16", "internalType": "uint16"}, {"name": "statValue", "type": "uint16", "internalType": "uint16"}, {"name": "backgroundBonus", "type": "uint16", "internalType": "uint16"}, {"name": "classBonus", "type": "uint16", "internalType": "uint16"}, {"name": "subclassBonus", "type": "uint16", "internalType": "uint16"}, {"name": "total", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IDuelS2Types.HeroTurnScore"}, {"name": "hero2Score", "type": "tuple", "components": [{"name": "roll", "type": "uint8", "internalType": "uint8"}, {"name": "elementBonus", "type": "uint16", "internalType": "uint16"}, {"name": "statValue", "type": "uint16", "internalType": "uint16"}, {"name": "backgroundBonus", "type": "uint16", "internalType": "uint16"}, {"name": "classBonus", "type": "uint16", "internalType": "uint16"}, {"name": "subclassBonus", "type": "uint16", "internalType": "uint16"}, {"name": "total", "type": "uint16", "internalType": "uint16"}], "internalType": "struct IDuelS2Types.HeroTurnScore"}, {"name": "winnerHeroId", "type": "uint256", "internalType": "uint256"}, {"name": "winnerPlayer", "type": "address", "internalType": "address"}], "internalType": "struct IDuelS2Types.TurnResult[]"}], "stateMutability": "view"},
    {"name": "getHeroDuel", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player1", "type": "address", "internalType": "address"}, {"name": "player2", "type": "address", "internalType": "address"}, {"name": "player1DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "player2DuelEntry", "type": "uint256", "internalType": "uint256"}, {"name": "winner", "type": "address", "internalType": "address"}, {"name": "player1Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "player2Heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "player1ScoreChange", "type": "tuple", "components": [{"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "base", "type": "uint16", "internalType": "uint16"}, {"name": "streakBonus", "type": "uint32", "internalType": "uint32"}, {"name": "miscBonus", "type": "uint16", "internalType": "uint16"}, {"name": "diffBonus", "type": "uint32", "internalType": "uint32"}, {"name": "scoreBefore", "type": "uint64", "internalType": "uint64"}, {"name": "scoreAfter", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.PlayerScoreChange"}], "internalType": "struct IDuelS2Types.Duel"}], "stateMutability": "view"},
    {"name": "getHeroDuelCountForDay", "type": "function", "inputs": [{"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_duelType", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "getHeroDuelEntry", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getHeroLastTimePlayed", "type": "function", "inputs": [{"name": "_heroId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getHighestScore", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint64", "internalType": "uint64"}], "stateMutability": "view"},
    {"name": "getNumRanks", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "getPlayerDuelEntries", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry[]"}], "stateMutability": "view"},
    {"name": "getPlayerRank", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getPlayerScore", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint64", "internalType": "uint64"}], "stateMutability": "view"},
    {"name": "getPracticeEntry", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_rank", "type": "uint8", "internalType": "uint8"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getRankLevels", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint16[]", "internalType": "uint16[]"}], "stateMutability": "pure"},
    {"name": "getSeasonInfo", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTotalDuelEntries", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTotalDuels", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTotalOpenDuelEntries", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTotalPlayerDuels", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint64", "internalType": "uint64"}], "stateMutability": "view"},
    {"name": "getTotalPlayerWins", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint64", "internalType": "uint64"}], "stateMutability": "view"},
    {"name": "getWinStreaks", "type": "function", "inputs": [{"name": "_player", "type": "address", "internalType": "address"}, {"name": "_type", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getDuelEntryMatch", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}, {"name": "_randomDistance", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getFirstOpenEntry", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getNextDuelEntryMatch", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getPrevDuelEntryMatch", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getTreeNode", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_value", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "_parent", "type": "uint256", "internalType": "uint256"}, {"name": "_left", "type": "uint256", "internalType": "uint256"}, {"name": "_right", "type": "uint256", "internalType": "uint256"}, {"name": "_red", "type": "bool", "internalType": "bool"}, {"name": "_keyCount", "type": "uint256", "internalType": "uint256"}, {"name": "_count", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTreePrev", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_key", "type": "uint256", "internalType": "uint256"}, {"name": "_value", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "player", "type": "address", "internalType": "address"}, {"name": "heroes", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "startBlock", "type": "uint256", "internalType": "uint256"}, {"name": "heroPower", "type": "uint256", "internalType": "uint256"}, {"name": "score", "type": "uint256", "internalType": "uint256"}, {"name": "scoreAfter", "type": "uint256", "internalType": "uint256"}, {"name": "tokenFee", "type": "uint256", "internalType": "uint256"}, {"name": "duelId", "type": "uint256", "internalType": "uint256"}, {"name": "custom1", "type": "uint256", "internalType": "uint256"}, {"name": "custom2", "type": "uint256", "internalType": "uint256"}, {"name": "duelType", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "uint8"}, {"name": "winStreak", "type": "uint64", "internalType": "uint64"}, {"name": "loseStreak", "type": "uint64", "internalType": "uint64"}], "internalType": "struct IDuelS2Types.DuelEntry"}], "stateMutability": "view"},
    {"name": "getTreeValue", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_value", "type": "uint256", "internalType": "uint256"}, {"name": "_index", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "_key", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTreeValueIndex", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_key", "type": "uint256", "internalType": "uint256"}, {"name": "_value", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "matchDuelEntry", "type": "function", "inputs": [{"name": "_duelEntry1ID", "type": "uint256", "internalType": "uint256"}, {"name": "_duelEntry2ID", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "matchMake", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "matchMakeAdmin", "type": "function", "inputs": [{"name": "_lobby", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "acceptChallenge", "type": "function", "inputs": [{"name": "_duelId", "type": "uint256", "internalType": "uint256"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_background", "type": "uint8", "internalType": "uint8"}, {"name": "_stat", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startPracticeDuel", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_rank", "type": "uint8", "internalType": "uint8"}, {"name": "_background", "type": "uint8", "internalType": "uint8"}, {"name": "_stat", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "startPrivateDuel", "type": "function", "inputs": [{"name": "_type", "type": "uint256", "internalType": "uint256"}, {"name": "_heroIds", "type": "uint256[]", "internalType": "uint256[]"}, {"name": "_opponent", "type": "address", "internalType": "address"}, {"name": "_background", "type": "uint8", "internalType": "uint8"}, {"name": "_stat", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "init", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_statScienceAddress", "type": "address", "internalType": "address"}, {"name": "_raffleWholesale", "type": "address", "internalType": "address"}, {"name": "_dfkGold", "type": "address", "internalType": "address"}, {"name": "_powerToken", "type": "address", "internalType": "address"}, {"name": "_goldPot", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class DFKDuelS2(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def cancel_duel_entry(self, cred:Credentials, _id:uint256) -> TxReceipt:
        tx = self.contract.functions.cancelDuelEntry(_id)
        return self.send_transaction(tx, cred)

    def set_duel_complete_blocks(self, cred:Credentials, _blocks:uint16) -> TxReceipt:
        tx = self.contract.functions.setDuelCompleteBlocks(_blocks)
        return self.send_transaction(tx, cred)

    def set_gold_pot(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.setGoldPot(_address)
        return self.send_transaction(tx, cred)

    def set_lose_win_thresholds(self, cred:Credentials, _lose:uint64, _win:uint64) -> TxReceipt:
        tx = self.contract.functions.setLoseWinThresholds(_lose, _win)
        return self.send_transaction(tx, cred)

    def set_match_limit(self, cred:Credentials, _limit:uint256) -> TxReceipt:
        tx = self.contract.functions.setMatchLimit(_limit)
        return self.send_transaction(tx, cred)

    def set_match_maker(self, cred:Credentials, _address:address, _access:bool) -> TxReceipt:
        tx = self.contract.functions.setMatchMaker(_address, _access)
        return self.send_transaction(tx, cred)

    def set_match_threshold(self, cred:Credentials, _lobby:uint256, _threshold:uint256) -> TxReceipt:
        tx = self.contract.functions.setMatchThreshold(_lobby, _threshold)
        return self.send_transaction(tx, cred)

    def set_moderator(self, cred:Credentials, _address:address, _access:bool) -> TxReceipt:
        tx = self.contract.functions.setModerator(_address, _access)
        return self.send_transaction(tx, cred)

    def set_open_matchmaking(self, cred:Credentials, _open:bool) -> TxReceipt:
        tx = self.contract.functions.setOpenMatchmaking(_open)
        return self.send_transaction(tx, cred)

    def set_score(self, cred:Credentials, _address:address, _type:uint256, _score:uint64) -> TxReceipt:
        tx = self.contract.functions.setScore(_address, _type, _score)
        return self.send_transaction(tx, cred)

    def set_season_start_and_duration(self, cred:Credentials, _start:uint256, _duration:uint256) -> TxReceipt:
        tx = self.contract.functions.setSeasonStartAndDuration(_start, _duration)
        return self.send_transaction(tx, cred)

    def set_stat_science_address(self, cred:Credentials, _stat_science_address:address) -> TxReceipt:
        tx = self.contract.functions.setStatScienceAddress(_stat_science_address)
        return self.send_transaction(tx, cred)

    def set_timeout_threshold(self, cred:Credentials, _threshold:uint256) -> TxReceipt:
        tx = self.contract.functions.setTimeoutThreshold(_threshold)
        return self.send_transaction(tx, cred)

    def set_type_allowed(self, cred:Credentials, _lobby:uint256, _allowed:bool) -> TxReceipt:
        tx = self.contract.functions.setTypeAllowed(_lobby, _allowed)
        return self.send_transaction(tx, cred)

    def complete_duel(self, cred:Credentials, _duel_id:uint256) -> TxReceipt:
        tx = self.contract.functions.completeDuel(_duel_id)
        return self.send_transaction(tx, cred)

    def get_current_class_bonuses(self, block_identifier:BlockIdentifier = 'latest') -> List[List[uint8]]:
        return self.contract.functions.getCurrentClassBonuses().call(block_identifier=block_identifier)

    def complete_duel_admin(self, cred:Credentials, _duel_id:uint256) -> TxReceipt:
        tx = self.contract.functions.completeDuelAdmin(_duel_id)
        return self.send_transaction(tx, cred)

    def get_key_exists(self, _type:uint256, _key:uint256, _value:uint256, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.getKeyExists(_type, _key, _value).call(block_identifier=block_identifier)

    def enter_duel_lobby(self, cred:Credentials, _type:uint256, _hero_ids:Sequence[uint256], _token_fee:uint256, _background:uint8, _stat:uint8) -> TxReceipt:
        tx = self.contract.functions.enterDuelLobby(_type, _hero_ids, _token_fee, _background, _stat)
        return self.send_transaction(tx, cred)

    def get_active_duels(self, _address:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getActiveDuels(_address).call(block_identifier=block_identifier)

    def get_challenges(self, _profile:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getChallenges(_profile).call(block_identifier=block_identifier)

    def get_current_hero_score_duel_id(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getCurrentHeroScoreDuelId(_hero_id).call(block_identifier=block_identifier)

    def get_duel(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getDuel(_id).call(block_identifier=block_identifier)

    def get_duel_entry(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getDuelEntry(_id).call(block_identifier=block_identifier)

    def get_duel_history(self, _profile:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getDuelHistory(_profile).call(block_identifier=block_identifier)

    def get_duel_index_p1(self, _duel_id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getDuelIndexP1(_duel_id).call(block_identifier=block_identifier)

    def get_duel_rewards(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getDuelRewards(_id).call(block_identifier=block_identifier)

    def get_duel_turns(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getDuelTurns(_id).call(block_identifier=block_identifier)

    def get_hero_duel(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getHeroDuel(_hero_id).call(block_identifier=block_identifier)

    def get_hero_duel_count_for_day(self, _hero_ids:Sequence[uint256], _duel_type:uint256, block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getHeroDuelCountForDay(_hero_ids, _duel_type).call(block_identifier=block_identifier)

    def get_hero_duel_entry(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getHeroDuelEntry(_hero_id).call(block_identifier=block_identifier)

    def get_hero_last_time_played(self, _hero_id:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getHeroLastTimePlayed(_hero_id).call(block_identifier=block_identifier)

    def get_highest_score(self, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint64:
        return self.contract.functions.getHighestScore(_type).call(block_identifier=block_identifier)

    def get_num_ranks(self, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getNumRanks(_type).call(block_identifier=block_identifier)

    def get_player_duel_entries(self, _profile:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getPlayerDuelEntries(_profile).call(block_identifier=block_identifier)

    def get_player_rank(self, _profile:address, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getPlayerRank(_profile, _type).call(block_identifier=block_identifier)

    def get_player_score(self, _profile:address, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint64:
        return self.contract.functions.getPlayerScore(_profile, _type).call(block_identifier=block_identifier)

    def get_practice_entry(self, _type:uint256, _rank:uint8, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getPracticeEntry(_type, _rank).call(block_identifier=block_identifier)

    def get_rank_levels(self, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> List[uint16]:
        return self.contract.functions.getRankLevels(_type).call(block_identifier=block_identifier)

    def get_season_info(self, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256]:
        return self.contract.functions.getSeasonInfo().call(block_identifier=block_identifier)

    def get_total_duel_entries(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getTotalDuelEntries().call(block_identifier=block_identifier)

    def get_total_duels(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getTotalDuels().call(block_identifier=block_identifier)

    def get_total_open_duel_entries(self, _lobby:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getTotalOpenDuelEntries(_lobby).call(block_identifier=block_identifier)

    def get_total_player_duels(self, _profile:address, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint64:
        return self.contract.functions.getTotalPlayerDuels(_profile, _type).call(block_identifier=block_identifier)

    def get_total_player_wins(self, _profile:address, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint64:
        return self.contract.functions.getTotalPlayerWins(_profile, _type).call(block_identifier=block_identifier)

    def get_win_streaks(self, _player:address, _type:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getWinStreaks(_player, _type).call(block_identifier=block_identifier)

    def get_duel_entry_match(self, _id:uint256, _random_distance:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getDuelEntryMatch(_id, _random_distance).call(block_identifier=block_identifier)

    def get_first_open_entry(self, _lobby:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getFirstOpenEntry(_lobby).call(block_identifier=block_identifier)

    def get_next_duel_entry_match(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getNextDuelEntryMatch(_id).call(block_identifier=block_identifier)

    def get_prev_duel_entry_match(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getPrevDuelEntryMatch(_id).call(block_identifier=block_identifier)

    def get_tree_node(self, _type:uint256, _value:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint256, bool, uint256, uint256]:
        return self.contract.functions.getTreeNode(_type, _value).call(block_identifier=block_identifier)

    def get_tree_prev(self, _type:uint256, _key:uint256, _value:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getTreePrev(_type, _key, _value).call(block_identifier=block_identifier)

    def get_tree_value(self, _type:uint256, _value:uint256, _index:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getTreeValue(_type, _value, _index).call(block_identifier=block_identifier)

    def get_tree_value_index(self, _type:uint256, _key:uint256, _value:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getTreeValueIndex(_type, _key, _value).call(block_identifier=block_identifier)

    def match_duel_entry(self, cred:Credentials, _duel_entry1_id:uint256, _duel_entry2_id:uint256) -> TxReceipt:
        tx = self.contract.functions.matchDuelEntry(_duel_entry1_id, _duel_entry2_id)
        return self.send_transaction(tx, cred)

    def match_make(self, cred:Credentials, _lobby:uint256) -> TxReceipt:
        tx = self.contract.functions.matchMake(_lobby)
        return self.send_transaction(tx, cred)

    def match_make_admin(self, cred:Credentials, _lobby:uint256) -> TxReceipt:
        tx = self.contract.functions.matchMakeAdmin(_lobby)
        return self.send_transaction(tx, cred)

    def accept_challenge(self, cred:Credentials, _duel_id:uint256, _hero_ids:Sequence[uint256], _background:uint8, _stat:uint8) -> TxReceipt:
        tx = self.contract.functions.acceptChallenge(_duel_id, _hero_ids, _background, _stat)
        return self.send_transaction(tx, cred)

    def start_practice_duel(self, cred:Credentials, _type:uint256, _hero_ids:Sequence[uint256], _rank:uint8, _background:uint8, _stat:uint8) -> TxReceipt:
        tx = self.contract.functions.startPracticeDuel(_type, _hero_ids, _rank, _background, _stat)
        return self.send_transaction(tx, cred)

    def start_private_duel(self, cred:Credentials, _type:uint256, _hero_ids:Sequence[uint256], _opponent:address, _background:uint8, _stat:uint8) -> TxReceipt:
        tx = self.contract.functions.startPrivateDuel(_type, _hero_ids, _opponent, _background, _stat)
        return self.send_transaction(tx, cred)

    def init(self, cred:Credentials, _hero_core_address:address, _stat_science_address:address, _raffle_wholesale:address, _dfk_gold:address, _power_token:address, _gold_pot:address) -> TxReceipt:
        tx = self.contract.functions.init(_hero_core_address, _stat_science_address, _raffle_wholesale, _dfk_gold, _power_token, _gold_pot)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xE959cbddB8616BDFFa5464279664CCbb9EA10317",
    "sd": "0x4aBb1cDe7a0C55850495E80E1806993b1B92F742"
}

ABI = """[
    {"type": "constructor", "inputs": [{"name": "_petCore", "type": "address", "internalType": "address"}], "stateMutability": "nonpayable"},
    {"name": "PetReleased", "type": "event", "inputs": [{"name": "petId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "releaseReason", "type": "string", "indexed": true, "internalType": "string"}, {"name": "releaseTime", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "getProfileReleasedPet", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}, {"name": "_index", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "internalType": "struct ReleasedPet"}], "stateMutability": "view"},
    {"name": "getProfileReleasedPets", "type": "function", "inputs": [{"name": "_profile", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "internalType": "struct ReleasedPet[]"}], "stateMutability": "view"},
    {"name": "getReleasedPet", "type": "function", "inputs": [{"name": "_petId", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "internalType": "struct ReleasedPet"}], "stateMutability": "view"},
    {"name": "getReleasedPets", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "internalType": "struct ReleasedPet[]"}], "stateMutability": "view"},
    {"name": "petCore", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IPetCore"}], "stateMutability": "view"},
    {"name": "petToReleasedPet", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "profileReleasedPets", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "releasePet", "type": "function", "inputs": [{"name": "_holder", "type": "address", "internalType": "address"}, {"name": "_petId", "type": "uint256", "internalType": "uint256"}, {"name": "_releaseReason", "type": "string", "internalType": "string"}, {"name": "_previousOwner", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "releasedPets", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "petId", "type": "uint256", "internalType": "uint256"}, {"name": "releaseTime", "type": "uint64", "internalType": "uint64"}, {"name": "releaseReason", "type": "string", "internalType": "string"}, {"name": "previousOwner", "type": "address", "internalType": "address"}], "stateMutability": "view"}
]
"""     

class Pasture(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def get_profile_released_pet(self, _profile:address, _index:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getProfileReleasedPet(_profile, _index).call(block_identifier=block_identifier)

    def get_profile_released_pets(self, _profile:address, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getProfileReleasedPets(_profile).call(block_identifier=block_identifier)

    def get_released_pet(self, _pet_id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getReleasedPet(_pet_id).call(block_identifier=block_identifier)

    def get_released_pets(self, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getReleasedPets().call(block_identifier=block_identifier)

    def pet_core(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.petCore().call(block_identifier=block_identifier)

    def pet_to_released_pet(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.petToReleasedPet(a).call(block_identifier=block_identifier)

    def profile_released_pets(self, a:address, b:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint64, string, address]:
        return self.contract.functions.profileReleasedPets(a, b).call(block_identifier=block_identifier)

    def release_pet(self, cred:Credentials, _holder:address, _pet_id:uint256, _release_reason:string, _previous_owner:address) -> TxReceipt:
        tx = self.contract.functions.releasePet(_holder, _pet_id, _release_reason, _previous_owner)
        return self.send_transaction(tx, cred)

    def released_pets(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256, uint64, string, address]:
        return self.contract.functions.releasedPets(a).call(block_identifier=block_identifier)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x0f85fdf6c561C42d6b46d0E27ea6Aa9Bf9476B3f",
    "sd": "0x3Eab8a8F71dDA3e6c907C74302B734805C4f3278"
}

ABI = """[
    {"name": "ItemAdded", "type": "event", "inputs": [{"name": "item", "type": "address", "internalType": "address", "indexed": true}, {"name": "startingPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "minPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64", "indexed": false}], "anonymous": false},
    {"name": "ItemTraded", "type": "event", "inputs": [{"name": "player", "type": "address", "internalType": "address", "indexed": true}, {"name": "boughtItem", "type": "address", "internalType": "address", "indexed": false}, {"name": "boughtQty", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "soldItem", "type": "address", "internalType": "address", "indexed": false}, {"name": "soldQty", "type": "uint256", "internalType": "uint256", "indexed": false}], "anonymous": false},
    {"name": "ItemUpdated", "type": "event", "inputs": [{"name": "item", "type": "address", "internalType": "address", "indexed": true}, {"name": "currentPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "minPrice", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256", "indexed": false}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64", "indexed": false}, {"name": "status", "type": "uint8", "internalType": "uint8", "indexed": false}], "anonymous": false},
    {"name": "addTradeItem", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}, {"name": "_startingPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_sellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "_decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "_priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "addressToTradeItemId", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "buyItem", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}, {"name": "_quantity", "type": "uint256", "internalType": "uint256"}, {"name": "_maxPrice", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getNextPrice", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}, {"name": "_quantity", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getTradeItem", "type": "function", "inputs": [{"name": "_id", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "item", "type": "address", "internalType": "address"}, {"name": "currentPrice", "type": "uint256", "internalType": "uint256"}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}, {"name": "lastPurchaseTimestamp", "type": "uint64", "internalType": "uint64"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct TraderTypes.TradeItem"}], "stateMutability": "view"},
    {"name": "getTradeItemByAddress", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "item", "type": "address", "internalType": "address"}, {"name": "currentPrice", "type": "uint256", "internalType": "uint256"}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}, {"name": "lastPurchaseTimestamp", "type": "uint64", "internalType": "uint64"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct TraderTypes.TradeItem"}], "stateMutability": "view"},
    {"name": "getTradeItems", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "tuple[]", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "item", "type": "address", "internalType": "address"}, {"name": "currentPrice", "type": "uint256", "internalType": "uint256"}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}, {"name": "lastPurchaseTimestamp", "type": "uint64", "internalType": "uint64"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "internalType": "struct TraderTypes.TradeItem[]"}], "stateMutability": "view"},
    {"name": "sellItem", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}, {"name": "_quantity", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "tradeItems", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "item", "type": "address", "internalType": "address"}, {"name": "currentPrice", "type": "uint256", "internalType": "uint256"}, {"name": "playerSellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}, {"name": "lastPurchaseTimestamp", "type": "uint64", "internalType": "uint64"}, {"name": "status", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "updateTradeItem", "type": "function", "inputs": [{"name": "_itemAddress", "type": "address", "internalType": "address"}, {"name": "_currentPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_sellPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_minPrice", "type": "uint256", "internalType": "uint256"}, {"name": "_deltaPriceIncrease", "type": "uint256", "internalType": "uint256"}, {"name": "_decreaseRate", "type": "uint256", "internalType": "uint256"}, {"name": "_priceIncreaseDecay", "type": "uint64", "internalType": "uint64"}, {"name": "_status", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"}
]
"""     

class ItemGoldTraderV2(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def add_trade_item(self, cred:Credentials, _item_address:address, _starting_price:uint256, _sell_price:uint256, _min_price:uint256, _delta_price_increase:uint256, _decrease_rate:uint256, _price_increase_decay:uint64) -> TxReceipt:
        tx = self.contract.functions.addTradeItem(_item_address, _starting_price, _sell_price, _min_price, _delta_price_increase, _decrease_rate, _price_increase_decay)
        return self.send_transaction(tx, cred)

    def address_to_trade_item_id(self, a:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.addressToTradeItemId(a).call(block_identifier=block_identifier)

    def buy_item(self, cred:Credentials, _item_address:address, _quantity:uint256, _max_price:uint256) -> TxReceipt:
        tx = self.contract.functions.buyItem(_item_address, _quantity, _max_price)
        return self.send_transaction(tx, cred)

    def get_next_price(self, _item_address:address, _quantity:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getNextPrice(_item_address, _quantity).call(block_identifier=block_identifier)

    def get_trade_item(self, _id:uint256, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getTradeItem(_id).call(block_identifier=block_identifier)

    def get_trade_item_by_address(self, _item_address:address, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getTradeItemByAddress(_item_address).call(block_identifier=block_identifier)

    def get_trade_items(self, block_identifier:BlockIdentifier = 'latest') -> List[tuple]:
        return self.contract.functions.getTradeItems().call(block_identifier=block_identifier)

    def sell_item(self, cred:Credentials, _item_address:address, _quantity:uint256) -> TxReceipt:
        tx = self.contract.functions.sellItem(_item_address, _quantity)
        return self.send_transaction(tx, cred)

    def trade_items(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, address, uint256, uint256, uint256, uint256, uint256, uint64, uint64, uint8]:
        return self.contract.functions.tradeItems(a).call(block_identifier=block_identifier)

    def update_trade_item(self, cred:Credentials, _item_address:address, _current_price:uint256, _sell_price:uint256, _min_price:uint256, _delta_price_increase:uint256, _decrease_rate:uint256, _price_increase_decay:uint64, _status:uint8) -> TxReceipt:
        tx = self.contract.functions.updateTradeItem(_item_address, _current_price, _sell_price, _min_price, _delta_price_increase, _decrease_rate, _price_increase_decay, _status)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0xc32A0e963e50AAAED273A75425fC39902b0d0b3b",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "RecipeSet", "type": "event", "inputs": [{"name": "stoneAddress", "type": "address", "indexed": true, "internalType": "address"}, {"name": "requiredResources", "type": "address[]", "indexed": false, "internalType": "address[]"}, {"name": "requiredQuantities", "type": "uint32[]", "indexed": false, "internalType": "uint32[]"}, {"name": "active", "type": "bool", "indexed": false, "internalType": "bool"}], "anonymous": false},
    {"name": "ShopSetUp", "type": "event", "inputs": [{"name": "workingUntil", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "awayUntil", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "StoneCarved", "type": "event", "inputs": [{"name": "player", "type": "address", "indexed": true, "internalType": "address"}, {"name": "stoneAddress", "type": "address", "indexed": false, "internalType": "address"}, {"name": "quantity", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "adminSetupShop", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "awayUntil", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "carveStone", "type": "function", "inputs": [{"name": "_stoneAddress", "type": "address", "internalType": "address"}, {"name": "_quantity", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "getAvailability", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}, {"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getRecipe", "type": "function", "inputs": [{"name": "stoneAddress", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "tuple", "components": [{"name": "requiredResources", "type": "address[]", "internalType": "address[]"}, {"name": "requiredQuantities", "type": "uint32[]", "internalType": "uint32[]"}, {"name": "active", "type": "bool", "internalType": "bool"}], "internalType": "struct StoneCarver.Recipe"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "minClosedTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "minOpenTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "pause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "recipes", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "active", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "setRecipe", "type": "function", "inputs": [{"name": "_stoneAddress", "type": "address", "internalType": "address"}, {"name": "_requiredResources", "type": "address[]", "internalType": "address[]"}, {"name": "_requiredQuantities", "type": "uint32[]", "internalType": "uint32[]"}, {"name": "_active", "type": "bool", "internalType": "bool"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTimes", "type": "function", "inputs": [{"name": "_minOpenTime", "type": "uint256", "internalType": "uint256"}, {"name": "_varOpenTime", "type": "uint256", "internalType": "uint256"}, {"name": "_minClosedTime", "type": "uint256", "internalType": "uint256"}, {"name": "_varClosedTime", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setUpShop", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "unpause", "type": "function", "inputs": [], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "varClosedTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "varOpenTime", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "workingUntil", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"}
]
"""     

class StoneCarver(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def admin_setup_shop(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.adminSetupShop()
        return self.send_transaction(tx, cred)

    def away_until(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.awayUntil().call(block_identifier=block_identifier)

    def carve_stone(self, cred:Credentials, _stone_address:address, _quantity:uint256) -> TxReceipt:
        tx = self.contract.functions.carveStone(_stone_address, _quantity)
        return self.send_transaction(tx, cred)

    def get_availability(self, block_identifier:BlockIdentifier = 'latest') -> Tuple[uint256, uint256]:
        return self.contract.functions.getAvailability().call(block_identifier=block_identifier)

    def get_recipe(self, stone_address:address, block_identifier:BlockIdentifier = 'latest') -> tuple:
        return self.contract.functions.getRecipe(stone_address).call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.initialize()
        return self.send_transaction(tx, cred)

    def min_closed_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.minClosedTime().call(block_identifier=block_identifier)

    def min_open_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.minOpenTime().call(block_identifier=block_identifier)

    def pause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.pause()
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def recipes(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.recipes(a).call(block_identifier=block_identifier)

    def set_recipe(self, cred:Credentials, _stone_address:address, _required_resources:Sequence[address], _required_quantities:Sequence[uint32], _active:bool) -> TxReceipt:
        tx = self.contract.functions.setRecipe(_stone_address, _required_resources, _required_quantities, _active)
        return self.send_transaction(tx, cred)

    def set_times(self, cred:Credentials, _min_open_time:uint256, _var_open_time:uint256, _min_closed_time:uint256, _var_closed_time:uint256) -> TxReceipt:
        tx = self.contract.functions.setTimes(_min_open_time, _var_open_time, _min_closed_time, _var_closed_time)
        return self.send_transaction(tx, cred)

    def set_up_shop(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.setUpShop()
        return self.send_transaction(tx, cred)

    def unpause(self, cred:Credentials) -> TxReceipt:
        tx = self.contract.functions.unpause()
        return self.send_transaction(tx, cred)

    def var_closed_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.varClosedTime().call(block_identifier=block_identifier)

    def var_open_time(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.varOpenTime().call(block_identifier=block_identifier)

    def working_until(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.workingUntil().call(block_identifier=block_identifier)



from ..abi_multi_contract_wrapper import ABIMultiContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": None,
    "sd": None
}

ABI = """[
    {"name": "Approval", "type": "event", "inputs": [{"name": "owner", "type": "address", "internalType": "address", "indexed": true}, {"name": "spender", "type": "address", "internalType": "address", "indexed": true}, {"name": "value", "type": "uint256", "internalType": "uint256", "indexed": false}], "anonymous": false},
    {"name": "Transfer", "type": "event", "inputs": [{"name": "from", "type": "address", "internalType": "address", "indexed": true}, {"name": "to", "type": "address", "internalType": "address", "indexed": true}, {"name": "value", "type": "uint256", "internalType": "uint256", "indexed": false}], "anonymous": false},
    {"name": "allowance", "type": "function", "inputs": [{"name": "owner", "type": "address", "internalType": "address"}, {"name": "spender", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "approve", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "balanceOf", "type": "function", "inputs": [{"name": "account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "burn", "type": "function", "inputs": [{"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "burnFrom", "type": "function", "inputs": [{"name": "account", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "decimals", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint8", "internalType": "uint8"}], "stateMutability": "view"},
    {"name": "decreaseAllowance", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "subtractedValue", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "increaseAllowance", "type": "function", "inputs": [{"name": "spender", "type": "address", "internalType": "address"}, {"name": "addedValue", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "mint", "type": "function", "inputs": [{"name": "to", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "name", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "supportsInterface", "type": "function", "inputs": [{"name": "interfaceId", "type": "bytes4", "internalType": "bytes4"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "symbol", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "string", "internalType": "string"}], "stateMutability": "view"},
    {"name": "totalSupply", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "transfer", "type": "function", "inputs": [{"name": "recipient", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"},
    {"name": "transferFrom", "type": "function", "inputs": [{"name": "sender", "type": "address", "internalType": "address"}, {"name": "recipient", "type": "address", "internalType": "address"}, {"name": "amount", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "nonpayable"}
]
"""     

class ERC20(ABIMultiContractWrapper):
    def __init__(self, rpc:str):
        super().__init__(abi=ABI, rpc=rpc)

    def allowance(self, contract_address:address, owner:address, spender:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.allowance(owner, spender).call()

    def approve(self, cred:Credentials, contract_address:address, spender:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.approve(spender, amount)
        return self.send_transaction(tx, cred)

    def balance_of(self, contract_address:address, account:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.balanceOf(account).call()

    def burn(self, cred:Credentials, contract_address:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.burn(amount)
        return self.send_transaction(tx, cred)

    def burn_from(self, cred:Credentials, contract_address:address, account:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.burnFrom(account, amount)
        return self.send_transaction(tx, cred)

    def decimals(self, contract_address:address, block_identifier:BlockIdentifier = 'latest') -> uint8:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.decimals().call()

    def decrease_allowance(self, cred:Credentials, contract_address:address, spender:address, subtracted_value:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.decreaseAllowance(spender, subtracted_value)
        return self.send_transaction(tx, cred)

    def increase_allowance(self, cred:Credentials, contract_address:address, spender:address, added_value:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.increaseAllowance(spender, added_value)
        return self.send_transaction(tx, cred)

    def mint(self, cred:Credentials, contract_address:address, to:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.mint(to, amount)
        return self.send_transaction(tx, cred)

    def name(self, contract_address:address, block_identifier:BlockIdentifier = 'latest') -> string:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.name().call()

    def supports_interface(self, contract_address:address, interface_id:bytes4, block_identifier:BlockIdentifier = 'latest') -> bool:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.supportsInterface(interface_id).call()

    def symbol(self, contract_address:address, block_identifier:BlockIdentifier = 'latest') -> string:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.symbol().call()

    def total_supply(self, contract_address:address, block_identifier:BlockIdentifier = 'latest') -> uint256:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        return contract.functions.totalSupply().call()

    def transfer(self, cred:Credentials, contract_address:address, recipient:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.transfer(recipient, amount)
        return self.send_transaction(tx, cred)

    def transfer_from(self, cred:Credentials, contract_address:address, sender:address, recipient:address, amount:uint256) -> TxReceipt:
        contract = self.get_custom_contract(contract_address, abi=self.abi)
        tx = contract.functions.transferFrom(sender, recipient, amount)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x3C351E1afdd1b1BC44e931E12D4E05D6125eaeCa",
    "sd": "0x9e987E5E9aB872598f601BE4aCC5ac23F484845E"
}

ABI = """[
    {"type": "constructor", "inputs": [{"name": "_factory", "type": "address", "internalType": "address"}, {"name": "_WETH", "type": "address", "internalType": "address"}], "stateMutability": "nonpayable"},
    {"name": "WETH", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "addLiquidity", "type": "function", "inputs": [{"name": "tokenA", "type": "address", "internalType": "address"}, {"name": "tokenB", "type": "address", "internalType": "address"}, {"name": "amountADesired", "type": "uint256", "internalType": "uint256"}, {"name": "amountBDesired", "type": "uint256", "internalType": "uint256"}, {"name": "amountAMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountBMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountA", "type": "uint256", "internalType": "uint256"}, {"name": "amountB", "type": "uint256", "internalType": "uint256"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "addLiquidityETH", "type": "function", "inputs": [{"name": "token", "type": "address", "internalType": "address"}, {"name": "amountTokenDesired", "type": "uint256", "internalType": "uint256"}, {"name": "amountTokenMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountETHMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountToken", "type": "uint256", "internalType": "uint256"}, {"name": "amountETH", "type": "uint256", "internalType": "uint256"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}], "stateMutability": "payable"},
    {"name": "factory", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "getAmountIn", "type": "function", "inputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}, {"name": "reserveIn", "type": "uint256", "internalType": "uint256"}, {"name": "reserveOut", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "getAmountOut", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "reserveIn", "type": "uint256", "internalType": "uint256"}, {"name": "reserveOut", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "getAmountsIn", "type": "function", "inputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "getAmountsOut", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "view"},
    {"name": "quote", "type": "function", "inputs": [{"name": "amountA", "type": "uint256", "internalType": "uint256"}, {"name": "reserveA", "type": "uint256", "internalType": "uint256"}, {"name": "reserveB", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountB", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "removeLiquidity", "type": "function", "inputs": [{"name": "tokenA", "type": "address", "internalType": "address"}, {"name": "tokenB", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountAMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountBMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountA", "type": "uint256", "internalType": "uint256"}, {"name": "amountB", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "removeLiquidityETH", "type": "function", "inputs": [{"name": "token", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountTokenMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountETHMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountToken", "type": "uint256", "internalType": "uint256"}, {"name": "amountETH", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "removeLiquidityETHSupportingFeeOnTransferTokens", "type": "function", "inputs": [{"name": "token", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountTokenMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountETHMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amountETH", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "removeLiquidityETHWithPermit", "type": "function", "inputs": [{"name": "token", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountTokenMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountETHMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}, {"name": "approveMax", "type": "bool", "internalType": "bool"}, {"name": "v", "type": "uint8", "internalType": "uint8"}, {"name": "r", "type": "bytes32", "internalType": "bytes32"}, {"name": "s", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "amountToken", "type": "uint256", "internalType": "uint256"}, {"name": "amountETH", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "removeLiquidityETHWithPermitSupportingFeeOnTransferTokens", "type": "function", "inputs": [{"name": "token", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountTokenMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountETHMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}, {"name": "approveMax", "type": "bool", "internalType": "bool"}, {"name": "v", "type": "uint8", "internalType": "uint8"}, {"name": "r", "type": "bytes32", "internalType": "bytes32"}, {"name": "s", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "amountETH", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "removeLiquidityWithPermit", "type": "function", "inputs": [{"name": "tokenA", "type": "address", "internalType": "address"}, {"name": "tokenB", "type": "address", "internalType": "address"}, {"name": "liquidity", "type": "uint256", "internalType": "uint256"}, {"name": "amountAMin", "type": "uint256", "internalType": "uint256"}, {"name": "amountBMin", "type": "uint256", "internalType": "uint256"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}, {"name": "approveMax", "type": "bool", "internalType": "bool"}, {"name": "v", "type": "uint8", "internalType": "uint8"}, {"name": "r", "type": "bytes32", "internalType": "bytes32"}, {"name": "s", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "amountA", "type": "uint256", "internalType": "uint256"}, {"name": "amountB", "type": "uint256", "internalType": "uint256"}], "stateMutability": "nonpayable"},
    {"name": "swapETHForExactTokens", "type": "function", "inputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "payable"},
    {"name": "swapExactETHForTokens", "type": "function", "inputs": [{"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "payable"},
    {"name": "swapExactETHForTokensSupportingFeeOnTransferTokens", "type": "function", "inputs": [{"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "payable"},
    {"name": "swapExactTokensForETH", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "nonpayable"},
    {"name": "swapExactTokensForETHSupportingFeeOnTransferTokens", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "swapExactTokensForTokens", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "nonpayable"},
    {"name": "swapExactTokensForTokensSupportingFeeOnTransferTokens", "type": "function", "inputs": [{"name": "amountIn", "type": "uint256", "internalType": "uint256"}, {"name": "amountOutMin", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "swapTokensForExactETH", "type": "function", "inputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}, {"name": "amountInMax", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "nonpayable"},
    {"name": "swapTokensForExactTokens", "type": "function", "inputs": [{"name": "amountOut", "type": "uint256", "internalType": "uint256"}, {"name": "amountInMax", "type": "uint256", "internalType": "uint256"}, {"name": "path", "type": "address[]", "internalType": "address[]"}, {"name": "to", "type": "address", "internalType": "address"}, {"name": "deadline", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "amounts", "type": "uint256[]", "internalType": "uint256[]"}], "stateMutability": "nonpayable"},
    {"type": "receive", "stateMutability": "payable"}
]
"""     

class UniswapV2Router(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def weth(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.WETH().call(block_identifier=block_identifier)

    def add_liquidity(self, cred:Credentials, token_a:address, token_b:address, amount_a_desired:uint256, amount_b_desired:uint256, amount_a_min:uint256, amount_b_min:uint256, to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.addLiquidity(token_a, token_b, amount_a_desired, amount_b_desired, amount_a_min, amount_b_min, to, deadline)
        return self.send_transaction(tx, cred)

    def add_liquidity_eth(self, cred:Credentials, token:address, amount_token_desired:uint256, amount_token_min:uint256, amount_eth_min:uint256, to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.addLiquidityETH(token, amount_token_desired, amount_token_min, amount_eth_min, to, deadline)
        return self.send_transaction(tx, cred)

    def factory(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.factory().call(block_identifier=block_identifier)

    def get_amount_in(self, amount_out:uint256, reserve_in:uint256, reserve_out:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getAmountIn(amount_out, reserve_in, reserve_out).call(block_identifier=block_identifier)

    def get_amount_out(self, amount_in:uint256, reserve_in:uint256, reserve_out:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.getAmountOut(amount_in, reserve_in, reserve_out).call(block_identifier=block_identifier)

    def get_amounts_in(self, amount_out:uint256, path:Sequence[address], block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getAmountsIn(amount_out, path).call(block_identifier=block_identifier)

    def get_amounts_out(self, amount_in:uint256, path:Sequence[address], block_identifier:BlockIdentifier = 'latest') -> List[uint256]:
        return self.contract.functions.getAmountsOut(amount_in, path).call(block_identifier=block_identifier)

    def quote(self, amount_a:uint256, reserve_a:uint256, reserve_b:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.quote(amount_a, reserve_a, reserve_b).call(block_identifier=block_identifier)

    def remove_liquidity(self, cred:Credentials, token_a:address, token_b:address, liquidity:uint256, amount_a_min:uint256, amount_b_min:uint256, to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.removeLiquidity(token_a, token_b, liquidity, amount_a_min, amount_b_min, to, deadline)
        return self.send_transaction(tx, cred)

    def remove_liquidity_eth(self, cred:Credentials, token:address, liquidity:uint256, amount_token_min:uint256, amount_eth_min:uint256, to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.removeLiquidityETH(token, liquidity, amount_token_min, amount_eth_min, to, deadline)
        return self.send_transaction(tx, cred)

    def remove_liquidity_eth_supporting_fee_on_transfer_tokens(self, cred:Credentials, token:address, liquidity:uint256, amount_token_min:uint256, amount_eth_min:uint256, to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.removeLiquidityETHSupportingFeeOnTransferTokens(token, liquidity, amount_token_min, amount_eth_min, to, deadline)
        return self.send_transaction(tx, cred)

    def remove_liquidity_eth_with_permit(self, cred:Credentials, token:address, liquidity:uint256, amount_token_min:uint256, amount_eth_min:uint256, to:address, deadline:uint256, approve_max:bool, v:uint8, r:bytes32, s:bytes32) -> TxReceipt:
        tx = self.contract.functions.removeLiquidityETHWithPermit(token, liquidity, amount_token_min, amount_eth_min, to, deadline, approve_max, v, r, s)
        return self.send_transaction(tx, cred)

    def remove_liquidity_eth_with_permit_supporting_fee_on_transfer_tokens(self, cred:Credentials, token:address, liquidity:uint256, amount_token_min:uint256, amount_eth_min:uint256, to:address, deadline:uint256, approve_max:bool, v:uint8, r:bytes32, s:bytes32) -> TxReceipt:
        tx = self.contract.functions.removeLiquidityETHWithPermitSupportingFeeOnTransferTokens(token, liquidity, amount_token_min, amount_eth_min, to, deadline, approve_max, v, r, s)
        return self.send_transaction(tx, cred)

    def remove_liquidity_with_permit(self, cred:Credentials, token_a:address, token_b:address, liquidity:uint256, amount_a_min:uint256, amount_b_min:uint256, to:address, deadline:uint256, approve_max:bool, v:uint8, r:bytes32, s:bytes32) -> TxReceipt:
        tx = self.contract.functions.removeLiquidityWithPermit(token_a, token_b, liquidity, amount_a_min, amount_b_min, to, deadline, approve_max, v, r, s)
        return self.send_transaction(tx, cred)

    def swap_eth_for_exact_tokens(self, cred:Credentials, amount_out:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapETHForExactTokens(amount_out, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_eth_for_tokens(self, cred:Credentials, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactETHForTokens(amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_eth_for_tokens_supporting_fee_on_transfer_tokens(self, cred:Credentials, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactETHForTokensSupportingFeeOnTransferTokens(amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_tokens_for_eth(self, cred:Credentials, amount_in:uint256, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactTokensForETH(amount_in, amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_tokens_for_eth_supporting_fee_on_transfer_tokens(self, cred:Credentials, amount_in:uint256, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactTokensForETHSupportingFeeOnTransferTokens(amount_in, amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_tokens_for_tokens(self, cred:Credentials, amount_in:uint256, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactTokensForTokens(amount_in, amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_exact_tokens_for_tokens_supporting_fee_on_transfer_tokens(self, cred:Credentials, amount_in:uint256, amount_out_min:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapExactTokensForTokensSupportingFeeOnTransferTokens(amount_in, amount_out_min, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_tokens_for_exact_eth(self, cred:Credentials, amount_out:uint256, amount_in_max:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapTokensForExactETH(amount_out, amount_in_max, path, to, deadline)
        return self.send_transaction(tx, cred)

    def swap_tokens_for_exact_tokens(self, cred:Credentials, amount_out:uint256, amount_in_max:uint256, path:Sequence[address], to:address, deadline:uint256) -> TxReceipt:
        tx = self.contract.functions.swapTokensForExactTokens(amount_out, amount_in_max, path, to, deadline)
        return self.send_transaction(tx, cred)



from ..abi_contract_wrapper import ABIContractWrapper
from ..solidity_types import *
from ..credentials import Credentials

CONTRACT_ADDRESS =     {
    "cv": "0x70908Fd7278aab183C7EfC4f3449184E98e2e305",
    "sd": "0x0000000000000000000000000000000000000000"
}

ABI = """[
    {"name": "CrystalAirdrop", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "createdBlock", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "CrystalDarkSummoned", "type": "event", "inputs": [{"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "summonerId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "generation", "type": "uint16", "indexed": false, "internalType": "uint16"}, {"name": "createdBlock", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "summonerTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "assistantTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "enhancementStone", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "CrystalOpen", "type": "event", "inputs": [{"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "heroId", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "CrystalSummoned", "type": "event", "inputs": [{"name": "crystalId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "owner", "type": "address", "indexed": true, "internalType": "address"}, {"name": "summonerId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "generation", "type": "uint16", "indexed": false, "internalType": "uint16"}, {"name": "createdBlock", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "summonerTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "assistantTears", "type": "uint8", "indexed": false, "internalType": "uint8"}, {"name": "enhancementStone", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "EnhancementStoneAdded", "type": "event", "inputs": [{"name": "atunementItemAddress", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "FeeAddressAdded", "type": "event", "inputs": [{"name": "feeAddress", "type": "address", "indexed": true, "internalType": "address"}, {"name": "feePercent", "type": "uint256", "indexed": true, "internalType": "uint256"}], "anonymous": false},
    {"name": "FeeDeferred", "type": "event", "inputs": [{"name": "source", "type": "address", "indexed": true, "internalType": "address"}, {"name": "from", "type": "address", "indexed": true, "internalType": "address"}, {"name": "to", "type": "address", "indexed": true, "internalType": "address"}, {"name": "token", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "timestamp", "type": "uint64", "indexed": false, "internalType": "uint64"}], "anonymous": false},
    {"name": "FeeDisbursed", "type": "event", "inputs": [{"name": "source", "type": "address", "indexed": true, "internalType": "address"}, {"name": "from", "type": "address", "indexed": true, "internalType": "address"}, {"name": "to", "type": "address", "indexed": true, "internalType": "address"}, {"name": "token", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "timestamp", "type": "uint64", "indexed": false, "internalType": "uint64"}], "anonymous": false},
    {"name": "FeeLockedBurned", "type": "event", "inputs": [{"name": "source", "type": "address", "indexed": true, "internalType": "address"}, {"name": "from", "type": "address", "indexed": true, "internalType": "address"}, {"name": "to", "type": "address", "indexed": true, "internalType": "address"}, {"name": "token", "type": "address", "indexed": false, "internalType": "address"}, {"name": "amount", "type": "uint256", "indexed": false, "internalType": "uint256"}, {"name": "timestamp", "type": "uint64", "indexed": false, "internalType": "uint64"}], "anonymous": false},
    {"name": "GlobalStartTimeSet", "type": "event", "inputs": [{"name": "startTime", "type": "uint256", "indexed": false, "internalType": "uint256"}], "anonymous": false},
    {"name": "Initialized", "type": "event", "inputs": [{"name": "version", "type": "uint8", "indexed": false, "internalType": "uint8"}], "anonymous": false},
    {"name": "Paused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "RoleAdminChanged", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "previousAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "newAdminRole", "type": "bytes32", "indexed": true, "internalType": "bytes32"}], "anonymous": false},
    {"name": "RoleGranted", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "RoleRevoked", "type": "event", "inputs": [{"name": "role", "type": "bytes32", "indexed": true, "internalType": "bytes32"}, {"name": "account", "type": "address", "indexed": true, "internalType": "address"}, {"name": "sender", "type": "address", "indexed": true, "internalType": "address"}], "anonymous": false},
    {"name": "Unpaused", "type": "event", "inputs": [{"name": "account", "type": "address", "indexed": false, "internalType": "address"}], "anonymous": false},
    {"name": "DEFAULT_ADMIN_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "MODERATOR_ROLE", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "activeEnhancementStones", "type": "function", "inputs": [{"name": "", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "addEnhancementStone", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "baseCooldown", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "baseSummonFee", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "calculateRarityBonusCost", "type": "function", "inputs": [{"name": "_rarityBonusCharges", "type": "uint8", "internalType": "uint8"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "pure"},
    {"name": "calculateSummoningCost", "type": "function", "inputs": [{"name": "_hero", "type": "tuple", "components": [{"name": "id", "type": "uint256", "internalType": "uint256"}, {"name": "summoningInfo", "type": "tuple", "components": [{"name": "summonedTime", "type": "uint256", "internalType": "uint256"}, {"name": "nextSummonTime", "type": "uint256", "internalType": "uint256"}, {"name": "summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "summons", "type": "uint32", "internalType": "uint32"}, {"name": "maxSummons", "type": "uint32", "internalType": "uint32"}], "internalType": "struct SummoningInfo"}, {"name": "info", "type": "tuple", "components": [{"name": "statGenes", "type": "uint256", "internalType": "uint256"}, {"name": "visualGenes", "type": "uint256", "internalType": "uint256"}, {"name": "rarity", "type": "uint8", "internalType": "enum Rarity"}, {"name": "shiny", "type": "bool", "internalType": "bool"}, {"name": "generation", "type": "uint16", "internalType": "uint16"}, {"name": "firstName", "type": "uint32", "internalType": "uint32"}, {"name": "lastName", "type": "uint32", "internalType": "uint32"}, {"name": "shinyStyle", "type": "uint8", "internalType": "uint8"}, {"name": "class", "type": "uint8", "internalType": "uint8"}, {"name": "subClass", "type": "uint8", "internalType": "uint8"}], "internalType": "struct HeroInfo"}, {"name": "state", "type": "tuple", "components": [{"name": "staminaFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "hpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "mpFullAt", "type": "uint256", "internalType": "uint256"}, {"name": "level", "type": "uint16", "internalType": "uint16"}, {"name": "xp", "type": "uint64", "internalType": "uint64"}, {"name": "currentQuest", "type": "address", "internalType": "address"}, {"name": "sp", "type": "uint8", "internalType": "uint8"}, {"name": "status", "type": "uint8", "internalType": "enum HeroStatus"}], "internalType": "struct HeroState"}, {"name": "stats", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hp", "type": "uint16", "internalType": "uint16"}, {"name": "mp", "type": "uint16", "internalType": "uint16"}, {"name": "stamina", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStats"}, {"name": "primaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStatGrowth"}, {"name": "secondaryStatGrowth", "type": "tuple", "components": [{"name": "strength", "type": "uint16", "internalType": "uint16"}, {"name": "intelligence", "type": "uint16", "internalType": "uint16"}, {"name": "wisdom", "type": "uint16", "internalType": "uint16"}, {"name": "luck", "type": "uint16", "internalType": "uint16"}, {"name": "agility", "type": "uint16", "internalType": "uint16"}, {"name": "vitality", "type": "uint16", "internalType": "uint16"}, {"name": "endurance", "type": "uint16", "internalType": "uint16"}, {"name": "dexterity", "type": "uint16", "internalType": "uint16"}, {"name": "hpSm", "type": "uint16", "internalType": "uint16"}, {"name": "hpRg", "type": "uint16", "internalType": "uint16"}, {"name": "hpLg", "type": "uint16", "internalType": "uint16"}, {"name": "mpSm", "type": "uint16", "internalType": "uint16"}, {"name": "mpRg", "type": "uint16", "internalType": "uint16"}, {"name": "mpLg", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroStatGrowth"}, {"name": "professions", "type": "tuple", "components": [{"name": "mining", "type": "uint16", "internalType": "uint16"}, {"name": "gardening", "type": "uint16", "internalType": "uint16"}, {"name": "foraging", "type": "uint16", "internalType": "uint16"}, {"name": "fishing", "type": "uint16", "internalType": "uint16"}], "internalType": "struct HeroProfessions"}], "internalType": "struct Hero"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "cooldownPerGen", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "feeAddresses", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "address", "internalType": "address"}], "stateMutability": "view"},
    {"name": "feePercents", "type": "function", "inputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "getRoleAdmin", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}], "outputs": [{"name": "", "type": "bytes32", "internalType": "bytes32"}], "stateMutability": "view"},
    {"name": "grantRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "hasRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "increasePerGen", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "increasePerSummon", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "uint256", "internalType": "uint256"}], "stateMutability": "view"},
    {"name": "initialize", "type": "function", "inputs": [{"name": "_crystalCoreAddress", "type": "address", "internalType": "address"}, {"name": "_heroCoreAddress", "type": "address", "internalType": "address"}, {"name": "_powerTokenAddress", "type": "address", "internalType": "address"}, {"name": "_gaiaTearsAddress", "type": "address", "internalType": "address"}, {"name": "_statScienceAddress", "type": "address", "internalType": "address"}, {"name": "_graveyard", "type": "address", "internalType": "address"}, {"name": "_flagStorage", "type": "address", "internalType": "address"}, {"name": "_meditationCircle", "type": "address", "internalType": "address"}, {"name": "_assistingAuction", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "paused", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"},
    {"name": "powerToken", "type": "function", "inputs": [], "outputs": [{"name": "", "type": "address", "internalType": "contract IPowerToken"}], "stateMutability": "view"},
    {"name": "removeEnhancementStone", "type": "function", "inputs": [{"name": "_address", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "renounceRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "revokeRole", "type": "function", "inputs": [{"name": "role", "type": "bytes32", "internalType": "bytes32"}, {"name": "account", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setFees", "type": "function", "inputs": [{"name": "_feeAddresses", "type": "address[]", "internalType": "address[]"}, {"name": "_feePercents", "type": "uint256[]", "internalType": "uint256[]"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setHeroCore", "type": "function", "inputs": [{"name": "_heroCoreAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setPowerToken", "type": "function", "inputs": [{"name": "_powerTokenAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setStatScience", "type": "function", "inputs": [{"name": "_statScienceAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setSummonFees", "type": "function", "inputs": [{"name": "_baseSummonFee", "type": "uint256", "internalType": "uint256"}, {"name": "_increasePerSummon", "type": "uint256", "internalType": "uint256"}, {"name": "_increasePerGen", "type": "uint256", "internalType": "uint256"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTears", "type": "function", "inputs": [{"name": "_tearsAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "setTokenUnlocker", "type": "function", "inputs": [{"name": "_tokenUnlockerAddress", "type": "address", "internalType": "address"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "summonCrystal", "type": "function", "inputs": [{"name": "_summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "_assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "_summonerTears", "type": "uint16", "internalType": "uint16"}, {"name": "_assistantTears", "type": "uint16", "internalType": "uint16"}, {"name": "_enhancementStone", "type": "address", "internalType": "address"}, {"name": "_rarityBonusCharges", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "summonCrystalWithLocked", "type": "function", "inputs": [{"name": "_summonerId", "type": "uint256", "internalType": "uint256"}, {"name": "_assistantId", "type": "uint256", "internalType": "uint256"}, {"name": "_summonerTears", "type": "uint16", "internalType": "uint16"}, {"name": "_assistantTears", "type": "uint16", "internalType": "uint16"}, {"name": "_enhancementStone", "type": "address", "internalType": "address"}, {"name": "_rarityBonusCharges", "type": "uint8", "internalType": "uint8"}], "outputs": [], "stateMutability": "nonpayable"},
    {"name": "supportsInterface", "type": "function", "inputs": [{"name": "interfaceId", "type": "bytes4", "internalType": "bytes4"}], "outputs": [{"name": "", "type": "bool", "internalType": "bool"}], "stateMutability": "view"}
]
"""     

class DarkSummoning(ABIContractWrapper):
    def __init__(self, chain_key:str, rpc:str):
        contract_address = CONTRACT_ADDRESS[chain_key]
        super().__init__(contract_address=contract_address, abi=ABI, rpc=rpc)

    def active_enhancement_stones(self, a:address, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.activeEnhancementStones(a).call(block_identifier=block_identifier)

    def add_enhancement_stone(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.addEnhancementStone(_address)
        return self.send_transaction(tx, cred)

    def base_cooldown(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.baseCooldown().call(block_identifier=block_identifier)

    def base_summon_fee(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.baseSummonFee().call(block_identifier=block_identifier)

    def calculate_rarity_bonus_cost(self, _rarity_bonus_charges:uint8, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.calculateRarityBonusCost(_rarity_bonus_charges).call(block_identifier=block_identifier)

    def calculate_summoning_cost(self, _hero:tuple, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.calculateSummoningCost(_hero).call(block_identifier=block_identifier)

    def cooldown_per_gen(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.cooldownPerGen().call(block_identifier=block_identifier)

    def fee_addresses(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.feeAddresses(a).call(block_identifier=block_identifier)

    def fee_percents(self, a:uint256, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.feePercents(a).call(block_identifier=block_identifier)

    def increase_per_gen(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.increasePerGen().call(block_identifier=block_identifier)

    def increase_per_summon(self, block_identifier:BlockIdentifier = 'latest') -> uint256:
        return self.contract.functions.increasePerSummon().call(block_identifier=block_identifier)

    def initialize(self, cred:Credentials, _crystal_core_address:address, _hero_core_address:address, _power_token_address:address, _gaia_tears_address:address, _stat_science_address:address, _graveyard:address, _flag_storage:address, _meditation_circle:address, _assisting_auction:address) -> TxReceipt:
        tx = self.contract.functions.initialize(_crystal_core_address, _hero_core_address, _power_token_address, _gaia_tears_address, _stat_science_address, _graveyard, _flag_storage, _meditation_circle, _assisting_auction)
        return self.send_transaction(tx, cred)

    def paused(self, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.paused().call(block_identifier=block_identifier)

    def power_token(self, block_identifier:BlockIdentifier = 'latest') -> address:
        return self.contract.functions.powerToken().call(block_identifier=block_identifier)

    def remove_enhancement_stone(self, cred:Credentials, _address:address) -> TxReceipt:
        tx = self.contract.functions.removeEnhancementStone(_address)
        return self.send_transaction(tx, cred)

    def set_fees(self, cred:Credentials, _fee_addresses:Sequence[address], _fee_percents:Sequence[uint256]) -> TxReceipt:
        tx = self.contract.functions.setFees(_fee_addresses, _fee_percents)
        return self.send_transaction(tx, cred)

    def set_hero_core(self, cred:Credentials, _hero_core_address:address) -> TxReceipt:
        tx = self.contract.functions.setHeroCore(_hero_core_address)
        return self.send_transaction(tx, cred)

    def set_power_token(self, cred:Credentials, _power_token_address:address) -> TxReceipt:
        tx = self.contract.functions.setPowerToken(_power_token_address)
        return self.send_transaction(tx, cred)

    def set_stat_science(self, cred:Credentials, _stat_science_address:address) -> TxReceipt:
        tx = self.contract.functions.setStatScience(_stat_science_address)
        return self.send_transaction(tx, cred)

    def set_summon_fees(self, cred:Credentials, _base_summon_fee:uint256, _increase_per_summon:uint256, _increase_per_gen:uint256) -> TxReceipt:
        tx = self.contract.functions.setSummonFees(_base_summon_fee, _increase_per_summon, _increase_per_gen)
        return self.send_transaction(tx, cred)

    def set_tears(self, cred:Credentials, _tears_address:address) -> TxReceipt:
        tx = self.contract.functions.setTears(_tears_address)
        return self.send_transaction(tx, cred)

    def set_token_unlocker(self, cred:Credentials, _token_unlocker_address:address) -> TxReceipt:
        tx = self.contract.functions.setTokenUnlocker(_token_unlocker_address)
        return self.send_transaction(tx, cred)

    def summon_crystal(self, cred:Credentials, _summoner_id:uint256, _assistant_id:uint256, _summoner_tears:uint16, _assistant_tears:uint16, _enhancement_stone:address, _rarity_bonus_charges:uint8) -> TxReceipt:
        tx = self.contract.functions.summonCrystal(_summoner_id, _assistant_id, _summoner_tears, _assistant_tears, _enhancement_stone, _rarity_bonus_charges)
        return self.send_transaction(tx, cred)

    def summon_crystal_with_locked(self, cred:Credentials, _summoner_id:uint256, _assistant_id:uint256, _summoner_tears:uint16, _assistant_tears:uint16, _enhancement_stone:address, _rarity_bonus_charges:uint8) -> TxReceipt:
        tx = self.contract.functions.summonCrystalWithLocked(_summoner_id, _assistant_id, _summoner_tears, _assistant_tears, _enhancement_stone, _rarity_bonus_charges)
        return self.send_transaction(tx, cred)

    def supports_interface(self, interface_id:bytes4, block_identifier:BlockIdentifier = 'latest') -> bool:
        return self.contract.functions.supportsInterface(interface_id).call(block_identifier=block_identifier)